import 'dart:async';
import 'package:http/src/base_client.dart' as http;
import 'package:http/src/request.dart' as http show Request;
import 'package:http/src/response.dart' as http show Response;
import 'request.dart';
import 'response.dart';

abstract class BaseRapier {
  http.BaseClient _inner;
  http.BaseClient get inner => _inner;
  final bool debug;

  BaseRapier(this._inner, {this.debug: false});

  void close() => _inner.close();
  Request get(String url, {Map<String, String> headers}) =>
      new Request('GET', url, headers: headers, rapier: this);

  Request delete(String url, {Map<String, String> headers}) =>
      new Request('DELETE', url, headers: headers, rapier: this);

  Request post(String url, {body, Map<String, String> headers}) =>
      new Request('POST', url, body: body, headers: headers, rapier: this);

  Request put(String url, {body, Map<String, String> headers}) =>
      new Request('PUT', url, body: body, headers: headers, rapier: this);

  Request patch(String url, {body, Map<String, String> headers}) =>
      new Request('PATCH', url, body: body, headers: headers, rapier: this);

  Request head(String url, {Map<String, String> headers}) =>
      new Request('HEAD', url, headers: headers, rapier: this);

  Future<Response> send(Request request) async {
    final rq = new http.Request(request.method, Uri.parse(request.url));
    final rs = await inner.send(rq);
    return new Response(await http.Response.fromStream(rs));
  }
}
