import 'dart:async';
import 'base_rapier.dart';
import 'response.dart';

final RegExp _query = new RegExp(r'\?([^$]+)$');

class Request {
  String _url;
  final BaseRapier rapier;
  var body = null;
  final Map<String, String> headers = {};
  final String method;
  String get url => _url;

  Request(this.method, this._url,
      {this.body: null, Map<String, String> headers: const {}, this.rapier}) {
    this.headers.addAll(headers ?? {});
  }

  Future<Response> send() => rapier.send(this);

  Request operator +(other) {
    if (other is Map)
      return this..headers.addAll(other);
    else
      throw new Exception('Cannot add ${other.runtimeType} to Request.');
  }

  Request operator *(other) {
    if (other is Map) {
      final match = _query.firstMatch(_url);

      if (match != null) {
        _url = _url.replaceAll(match[0], '');
      }

      String query = match != null ? match[1] : '';
      final newQuery = other.keys
          .map((key) => '$key=${Uri.encodeQueryComponent(other[key])}');

      if (query.isNotEmpty) {
        query += '&' + newQuery.join('&');
      } else
        query = newQuery.join('&');

      return this.._url = _url + '?$query';
    } else
      throw new Exception('Cannot multiply Request by a ${other.runtimeType}.');
  }
}
